unit unit_upload;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IdBaseComponent, IdComponent, IdTCPConnection,
  IdTCPClient, IdFTP, Buttons, IdIntercept, IdLogBase, IdLogEvent, IdFTPCommon;

type
  Tfrm_Upload = class(TForm)
    lb_FtpLog: TListBox;
    ftp_Upload: TIdFTP;
    btn_Upload: TBitBtn;
    btn_Exit: TBitBtn;
    gb_RedirectSite: TGroupBox;
    ed_RSDir: TEdit;
    Label1: TLabel;
    ed_RSUser: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    ed_RSPass: TEdit;
    ed_RSHost: TEdit;
    Label4: TLabel;
    ed_RSPort: TEdit;
    Label5: TLabel;
    gb_UT2003Server: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    ed_UTDir: TEdit;
    ed_UTUser: TEdit;
    ed_UTPass: TEdit;
    ed_UTHost: TEdit;
    ed_UTPort: TEdit;
    cb_RedirectSite: TCheckBox;
    cb_UT2003Server: TCheckBox;
    cb_UploadAfterCompress: TCheckBox;
    IdLogEvent1: TIdLogEvent;
    procedure btn_UploadClick(Sender: TObject);
    procedure ftp_UploadAfterClientLogin(Sender: TObject);
    procedure ftp_UploadStatus(ASender: TObject; const AStatus: TIdStatus;
      const AStatusText: String);
    procedure ftp_UploadDisconnected(Sender: TObject);
    procedure btn_ExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure IdLogEvent1Received(ASender: TComponent; const AText,
      AData: String);
    procedure IdLogEvent1Status(ASender: TComponent; const AText: String);
    procedure IdLogEvent1Sent(ASender: TComponent; const AText,
      AData: String);
  private
    { Private declarations }
  public
    StartUploading: boolean;
    procedure UploadRedirect();
    procedure UploadServer();
    function ExtToPath(filename: string): string;
  end;

var
  frm_Upload: Tfrm_Upload;
  isUTServer: bool;
  lastServer: integer;

implementation

uses unit_main;

{$R *.dfm}

procedure Tfrm_Upload.UploadRedirect();
begin
  lastServer := 1;
  isUTServer := false;
  ftp_Upload.Host := ed_RSHost.Text;
  ftp_Upload.Port := StrToIntDef(ed_RSPort.Text, 21);
  ftp_Upload.Username := ed_RSUser.Text;
  ftp_Upload.Password := ed_RSPass.Text;
  try
    ftp_Upload.Connect(true);
  except
  end;
end;

procedure Tfrm_Upload.UploadServer();
begin
  lastServer := 2;
  isUTServer := true;
  ftp_Upload.Host := ed_UTHost.Text;
  ftp_Upload.Port := StrToIntDef(ed_UTPort.Text, 21);
  ftp_Upload.Username := ed_UTUser.Text;
  ftp_Upload.Password := ed_UTPass.Text;
  try
    ftp_Upload.Connect(true);
  except
  end;
end;

function Tfrm_Upload.ExtToPath(filename: string): string;
begin
  filename := LowerCase(extractfileext(filename));
  if (filename = '.ut2') then Result := 'Maps'
  else if (filename = '.u') then Result := 'System'
  else if (filename = '.utx') then Result := 'Textures'
  else if (filename = '.uax') then Result := 'Sounds'
  else if (filename = '.ogg') then Result := 'Music'
  else if (filename = '.usx') then Result := 'StaticMeshes'
  else if (filename = '.ka') then Result := 'KarmaData'
  else if (filename = '.ukx') then Result := 'Animations'
end;

procedure Tfrm_Upload.btn_UploadClick(Sender: TObject);
begin
  Application.ProcessMessages;
  lb_FtpLog.Items.Clear;
  if (cb_RedirectSite.Checked) then UploadRedirect()
  else if (cb_UT2003Server.Checked) then UploadServer();
end;

procedure Tfrm_Upload.ftp_UploadAfterClientLogin(Sender: TObject);
var
  i, size, size2: integer;
  filename: string;
begin
  ftp_Upload.TransferType := ftBinary;
  ftp_Upload.SendCmd('TYPE I', 200);
  try
    lb_FtpLog.Items.AddStrings(ftp_Upload.LoginMsg.Text);
    lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
    if (not isUTServer) then ftp_Upload.ChangeDir(ed_RSDir.Text);
    for i := 0 to frm_Main.lv_Files.Items.Count-1 do begin
      Application.ProcessMessages;
      if (isUTServer) then  filename := frm_Main.lv_Files.Items[i].Caption
      else filename := frm_Main.lv_Files.Items[i].Caption+UZEXT;
      if (FileExists(filename)) then begin
        try
          if (isUTServer) then begin
            ftp_Upload.ChangeDir(ed_UTDir.Text+ExtToPath(filename)+'/');
          end;
          size := ftp_Upload.Size(ExtractFileName(filename));
          if (isUTServer) then
            size2 := StrToIntDef(frm_Main.lv_Files.Items[i].SubItems[0], 0)
          else
            size2 := StrToIntDef(frm_Main.lv_Files.Items[i].SubItems[1], 0);
          if (size <> size2) then begin
            if (size <> -1) then ftp_Upload.Delete(ExtractFileName(filename));
            ftp_UploadStatus(Sender, ftpTransfer, 'Uploading file: '+filename);
            ftp_Upload.Put(filename, ExtractFileName(filename));
          end
          else begin
            ftp_UploadStatus(Sender, ftpTransfer, 'File '+ExtractFileName(filename)+' already exists');
          end;
        except
          on E:Exception do begin
            lb_FtpLog.Items.Add(E.Message);
            lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
          end;
        end;
      end;
    end;
  finally
    if (ftp_Upload.Connected) then ftp_Upload.Quit;
  end;
end;

procedure Tfrm_Upload.ftp_UploadStatus(ASender: TObject;
  const AStatus: TIdStatus; const AStatusText: String);
begin
  //lb_FtpLog.Items.Add(AStatusText);
  //lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
end;

procedure Tfrm_Upload.ftp_UploadDisconnected(Sender: TObject);
begin
  if (lastServer = 1) then begin
    if (cb_UT2003Server.Checked) then UploadServer();
  end;
end;

procedure Tfrm_Upload.btn_ExitClick(Sender: TObject);
begin
  Close;
end;

procedure Tfrm_Upload.FormActivate(Sender: TObject);
begin
  if (StartUploading) then begin
    StartUploading := false;
    btn_Upload.Click;
  end;
end;

procedure Tfrm_Upload.IdLogEvent1Received(ASender: TComponent; const AText,
  AData: String);
begin
  lb_FtpLog.Items.Add(AData);
  lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
end;

procedure Tfrm_Upload.IdLogEvent1Status(ASender: TComponent;
  const AText: String);
begin
  lb_FtpLog.Items.Add(AText);
  lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
end;

procedure Tfrm_Upload.IdLogEvent1Sent(ASender: TComponent; const AText,
  AData: String);
begin
  lb_FtpLog.Items.Add(AData);
  lb_FtpLog.ItemIndex := lb_FtpLog.Items.Count-1;
end;

end.
